from onepassword_tools.lib.OnePasswordSSHKeyItem import OnePasswordSSHKeyItem
from onepassword_tools.lib.NewItemCommand import NewItemCommand, new_item_command_options, new_item_command_password
from onepassword_tools.lib.MiscUtils import generate_password
from onepassword_tools.lib.Crypto import generate_ssh_key
from onepassword_tools.lib.ConfigFile import ConfigFile
from getpass import getuser
from socket import gethostname
import click


def get_defaults():
    _defaults = {}
    config_file = ConfigFile()
    if '.' in __name__:
        command_name = __name__.rsplit('.', 1)[1]
    else:
        command_name = __name__
    command_config = config_file.get_section(command_name)

    # Grab from-host default value from config file or fallback to hostname
    if config_file.keys_exists(command_config, 'defaults', 'from-host'):
        _defaults['from-host'] = command_config['defaults']['from-host']
    else:
        _defaults['from-host'] = gethostname().encode('ascii', 'ignore').decode('ascii')

    # Grab from-user default value from config or fallback to current user name
    if config_file.keys_exists(command_config, 'defaults', 'from-user'):
        _defaults['from-user'] = command_config['defaults']['from-user']
    else:
        _defaults['from-user'] = getuser()
    return _defaults


defaults = get_defaults()


@click.command()
@new_item_command_options
@click.option('--from-user', help='User who is responsible from initiating the connection, default current user.', prompt=True, default=defaults['from-user'])
@click.option('--from-host', help='Host from where the SSH connection starts, default current hostname.', prompt=True, default=defaults['from-host'])
@click.option('--to-user', help='Remote user, prompted if empty.', prompt=True, required=True)
@click.option('--to-host', help='Remote server hostname, prompted if empty.', prompt=True, required=True)
@click.option('--to-ip', help='Remote server IP, prompted if empty.', prompt=True, default='')
@click.option('--to-port', help='Remote SSH Port, prompted if empty.', prompt=True, default='22')
@click.option('--to-host-abbreviated', help='Alias of the remote server hostname used to initialize connection.', required=False)
@click.option('--no-passphrase', help='Create ssh key without passphrase.', prompt=False, required=False, default=False, is_flag=True)
@click.option('--passphrase', help='Use this passphrase instead of an autogenerated one.')
@click.option('--passphrase-length', help='Length of the autogenerated passphrase.', default=50, type=int)
@click.option('--port', help='Remote port', required=False, default=22, type=int)
def new_ssh_key(from_user, from_host, to_user, to_host, to_ip, to_port, to_host_abbreviated, no_passphrase, passphrase, passphrase_length,
                port, account, notes, return_field, title, vault, do_not_ask_credentials):
    """Generates a new SSH key and store it in 1Password. Additional information are stored also to generate SSH config
    file when imported."""
    NewSSHKey(**locals()).run()


class NewSSHKey(NewItemCommand):

    database: str = None
    from_user: str = None
    from_host: str = None
    no_passphrase: bool = None
    onePasswordItemClass = OnePasswordSSHKeyItem
    passphrase: str = None
    passphrase_length: int = None
    port: str = None
    public_key: str
    private_ey: str
    tags = ['Clef SSH']
    titleTemplate = 'id_rsa.from__{from_user}@{from_host}__to__{to_user}@{to_host}'
    to_user: str = None
    to_host: str = None
    to_ip: str = None
    to_port: str = None
    to_host_abbreviated: str = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if not self.no_passphrase and not self.passphrase:
            self.passphrase = generate_password(self.passphrase_length)
        self.public_key, self.private_key = generate_ssh_key(passphrase=self.passphrase)
        self.public_key += " %s" % self.get_title()



from onepassword_local_search.models.Item import Item


class OnePasswordResult(Item):

    overview: str
    details: str
    id: str
    vaultUuid: str

    def __init__(self, response):
        for attr in response.keys():
            setattr(self, attr, response[attr])





#!/usr/bin/env python3
import click
import click_log
from onepassword_tools.ssh_add import ssh_add
from onepassword_tools.new_server_account import new_server_account
from onepassword_tools.new_ssh_key import new_ssh_key
from onepassword_tools.new_website_account import new_website_account
from onepassword_tools.new_database_account import new_database_account
from onepassword_tools.lib.Log import logger
click_log.basic_config(logger)


@click.group()
@click_log.simple_verbosity_option(logger)
def start():
    pass


start.add_command(ssh_add.ssh_add)
start.add_command(new_server_account.new_server_account)
start.add_command(new_ssh_key.new_ssh_key)
start.add_command(new_website_account.new_website_account)
start.add_command(new_database_account.new_database_account)

if __name__ == '__main__':
    start()



from onepassword_tools.lib.OnePasswordItem import OnePasswordItem
from urllib.parse import urlsplit,urlunsplit


class OnePasswordLoginItem(OnePasswordItem):

    item_type: str = 'LOGIN'
    notes: str = None
    password: str = None
    tags: [] = ['Compte utilisateur']
    url: str = None
    username: str = None
    tld_variations: str = None

    def __getitem__(self, item):
        return super().__getitem__(item)

    def __init__(self, **kwargs):
        super().__init__()
        for key, value in kwargs.items():
            if hasattr(self, key):
                self.__setattr__(key, value)

    def _request_object(self):
        """
        Return the dict object with the request sent to 1Password
        :return: dict
        """

        tld_variations = self.tld_variations.split(',')
        url_splitted = urlsplit(self.url)
        url_variations = [
            {
                "label": "",
                "primary": True,
                "href": self.url
            }
        ]

        for tld_variation in tld_variations:
            url_splitted_tld = url_splitted
            url_splitted_tld = url_splitted_tld._replace(netloc=url_splitted.hostname + '.' + tld_variation)
            url_variations.append(
                    {
                        "label": "",
                        "primary": False,
                        "href": urlunsplit(url_splitted_tld)
                    }
            )

        return {
            "sections": [],
            "urls": url_variations,
            "category": "LOGIN",
            "fields": [
                {
                    "id": "username",
                    "type": "STRING",
                    "purpose": "USERNAME",
                    "label": "username",
                    "value": self.username
                },
                {
                    "id": "password",
                    "type": "CONCEALED",
                    "purpose": "PASSWORD",
                    "label": "password",
                    "password_details": {
                        "strength": "TERRIBLE"
                    },
                    "value": self.password
                },
                {
                    "id": "notesPlain",
                    "type": "STRING",
                    "purpose": "NOTES",
                    "label": "notesPlain",
                    "value": self.get('notes')
                }
            ]
        }



from onepassword_tools.lib.OnePasswordItem import OnePasswordItem


class OnePasswordDatabaseItem(OnePasswordItem):

    database: str = None
    hostname: str = None
    item_type: str = 'Database'
    notes: str = None
    password: str = None
    port: str = None
    url: str = None
    username: str = None

    def __getitem__(self, item):
        return super().__getitem__(item)

    def __init__(self, **kwargs):
        super().__init__()
        for key, value in kwargs.items():
            if hasattr(self, key):
                self.__setattr__(key, value)

    def _request_object(self):
        """
        Return the dict object with the request sent to 1Password
        :return: dict
        """
        section = {
            "title": "",
            "name": "Section_%s" % self.opu.generate_op_section_uuid()
        }


        return {
            "sections": [
                section
            ],
            "category": "DATABASE",
            "fields": [
                {
                    "id": "notesPlain",
                    "type": "STRING",
                    "purpose": "NOTES",
                    "label": "notedPlain",
                    "value": self.get('notes')
                },
                {
                    "type": "MENU",
                    "id": "database_type",
                    "label": "type",
                    "value": ""
                },
                {
                    "type": "STRING",
                    "id": "hostname",
                    "value": self.hostname,
                    "label": "server"
                },
                {
                    "inputTraits": {
                        "keyboard": "NumberPad"
                    },
                    "type": "STRING",
                    "id": "port",
                    "value": self.port,
                    "label": "port"
                },
                {
                    "inputTraits": {
                        "autocapitalization": "none",
                        "autocorrection": "no"
                    },
                    "type": "string",
                    "id": "database",
                    "value": self.database,
                    "label": "database"
                },
                {
                    "type": "STRING",
                    "id": "username",
                    "value": self.username,
                    "label": "username"
                },
                {
                    "type": "concealed",
                    "id": "password",
                    "value": self.password,
                    "label": "password"
                },
                {
                    "type": "STRING",
                    "id": "sid",
                    "label": "SID",
                    "value": ""
                },
                {
                    "type": "STRING",
                    "id": "alias",
                    "label": "alias",
                    "value": ""
                },
                {
                    "type": "STRING",
                    "id": "options",
                    "label": "connection options",
                    "value": ""
                }
            ]
        }
